@@ -5,6 +5,8 @@
 const char *builtins[]=
 {
 	"joerc",
+		"-keepup	%k and %c status line escape sequences updated frequently\n"
+		"-noxon		Attempt to turn off ^S/^Q processing\n"
 		"-notite         Don't send terminal initialization and termination strings:\n"
 		"-assume_color	Assume terminal has ANSI color support even if termcap/terminfo entry\n"
 		"-assume_256color\n"
@@ -20,9 +22,12 @@ const char *builtins[]=
 		"-restore	Restore previous cursor position when files are opened\n"
 		"-search_prompting\n"
 		"-lmsg \\i%k%T%W%I%X %n %m%y%R %M %x\n"
-		"-rmsg  %S Row %4r Col %3c %t  Ctrl-K H for help\n"
+		"-rmsg  %S Row %4r Col %3c \n"
 		"-smsg ** Line %r Col %c Offset %o(0x%O) %e %a(0x%A) Width %w ** \n"
 		"-zmsg ** Line %r Col %c Offset %o(0x%O) ** \n"
+		"-xmsg \\i Joe's Own Editor %v (%b) ** Type \\bCtrl-K Q\\b to exit or \\bCtrl-K H\\b for help **\\i\n"
+		"-aborthint ^C\n"
+		"-helphint ^K H\n"
 		"-highlight\n"
 		"-istep 4\n"
 		":include ftyperc\n"
@@ -49,6 +54,7 @@ const char *builtins[]=
 		"mode,\"language\",rtn	V Language\n"
 		"mode,\"picture\",rtn	P picture %Zpicture%\n"
 		"mode,\"type\",rtn		F File type [%Ztype%]\n"
+		"mode,\"title\",rtn	C Context %Ztitle%\n"
 		"menu,\"more-options\",rtn	  % % More options...\n"
 		":defmenu more-options menu,\"root\",rtn\n"
 		"menu,\"^G\",rtn	% % ^G options\n"
@@ -121,6 +127,8 @@ const char *builtins[]=
 		":defmenu cursor menu,\"more-options\",rtn\n"
 		"mode,\"pg\",rtn	% % No. PgUp/PgDn lines %Zpg%\n"
 		"mode,\"mid\",rtn	C Center on scroll %Zmid%\n"
+		"mode,\"left\",rtn	L Columns to scroll left %Zleft%\n"
+		"mode,\"right\",rtn R Columns to scroll right %Zright%\n"
 		"mode,\"floatmouse\",rtn	% % Click past end %Zfloatmouse%\n"
 		"mode,\"rtbutton\",rtn	% % Right button %Zrtbutton%\n"
 		":defmenu marking menu,\"more-options\",rtn\n"
@@ -134,127 +142,155 @@ const char *builtins[]=
 		"mode,\"purify\",rtn	% % Clean up indents %Zpurify%\n"
 		"mode,\"spaces\",rtn	% % No tabs %Zspaces%\n"
 		"{Basic\n"
-		"\\i   Help Screen    \\|turn off with ^KH    more help with ESC . (^[.)              \\i\n"
-		"\\i \\i\\|\\uCURSOR\\u           \\|\\uGO TO\\u            \\|\\uBLOCK\\u      \\|\\uDELETE\\u   \\|\\uMISC\\u         \\|\\uEXIT\\u      \\|\\i \\i\n"
-		"\\i \\i\\|^B left ^F right \\|^U  prev. screen \\|^KB begin  \\|^D char. \\|^KJ reformat \\|^KX save  \\|\\i \\i\n"
-		"\\i \\i\\|^P up   ^N down  \\|^V  next screen  \\|^KK end    \\|^Y line  \\|^KA center   \\|^C  abort \\|\\i \\i\n"
-		"\\i \\i\\|^Z previous word \\|^A  beg. of line \\|^KM move   \\|^W >word \\|^T  options  \\|^KZ shell \\|\\i \\i\n"
-		"\\i \\i\\|^X next word     \\|^E  end of line  \\|^KC copy   \\|^O word< \\|^R  refresh  \\|\\uFILE\\u      \\|\\i \\i\n"
-		"\\i \\i\\|\\uSEARCH\\u           \\|^KU top of file  \\|^KW file   \\|^J >line \\|\\uSPELL\\u        \\|^KE edit  \\|\\i \\i\n"
-		"\\i \\i\\|^KF find text    \\|^KV end of file  \\|^KY delete \\|^_ undo  \\|^[N word     \\|^KR insert\\|\\i \\i\n"
-		"\\i \\i\\|^L  find next    \\|^KL to line No.  \\|^K/ filter \\|^^ redo  \\|^[L file     \\|^KD save  \\|\\i \\i\n"
-		"}\n"
-		"{Windows\n"
-		"\\i   Help Screen    \\|turn off with ^KH    prev. screen ^[,    next screen ^[.     \\i\n"
-		"\\i \\i\\|^KO Split the window in half           \\|F1 - F4 Pop-up shell window           \\|\\i \\i\n"
-		"\\i \\i\\|^KG Make current window bigger         \\|^KT Make current window smaller       \\|\\i \\i\n"
-		"\\i \\i\\|^KN Go to the window below             \\|^KP Go to the window above            \\|\\i \\i\n"
-		"\\i \\i\\|^[V Switch to next buffer              \\|^[U Switch to previous buffer         \\|\\i \\i\n"
-		"\\i \\i\\|^C  Eliminate the current window       \\|^KI Show all windows / Show one window\\|\\i \\i\n"
+		"\\|\\uREGION\\u        \\|\\uGO TO\\u           \\|\\uGO TO\\u           \\|\\uDELETE\\u    \\|\\uEXIT\\u       \\|\\uSEARCH\\u    \\| \n"
+		"\\|\\b^Arrow\\b Select \\|\\b^Z\\b Prev. word   \\|\\b^U/^V\\b PgUp/PgDn \\|\\b^D\\b Char.  \\|\\b^KX\\b Save   \\|\\b^KF\\b Find  \\| \n"
+		"\\|\\b^KB\\b Begin     \\|\\b^X\\b Next word    \\|\\uMISC\\u            \\|\\b^Y\\b Line   \\|\\b^C\\b  Abort  \\|\\b^L\\b  Next  \\| \n"
+		"\\|\\b^KK\\b End       \\|\\b^KU\\b Top of file \\|\\b^KJ\\b Paragraph   \\|\\b^W\\b >Word  \\|\\b^KQ\\b All    \\|\\uHELP\\u      \\| \n"
+		"\\|\\b^KC\\b Copy      \\|\\b^KV\\b End of file \\|\\b^KA\\b Center line \\|\\b^O\\b Word<  \\|\\uFILE\\u       \\|\\bEsc .\\b Next\\| \n"
+		"\\|\\b^KM\\b Move      \\|\\b^A\\b Beg. of line \\|\\b^K Space\\b Status \\|\\b^J\\b >Line  \\|\\b^KE\\b Edit   \\|\\bEsc ,\\b Prev\\| \n"
+		"\\|\\b^KW\\b File      \\|\\b^E\\b End of line  \\|\\uSPELL\\u           \\|\\b^[O\\b Line< \\|\\b^KR\\b Insert \\|\\b^KH\\b Off   \\| \n"
+		"\\|\\b^KY\\b Delete    \\|\\b^KL\\b To line no. \\|\\bEsc N\\b Word      \\|\\b^_\\b Undo   \\|\\b^KD\\b Save   \\|\\b^T\\b  Menu  \\| \n"
+		"\\|\\b^K/\\b Filter    \\|\\b^G\\b  Matching (  \\|\\bEsc L\\b File      \\|\\b^^\\b Redo   \\|\\b^K`\\b Revert \\|\\b\\b          \\| \n"
 		"}\n"
 		"{Advanced\n"
-		"\\i   Help Screen    \\|turn off with ^KH    prev. screen ^[,    next screen ^[.     \\i\n"
-		"\\i \\i\\|\\uMACROS\\u         \\|\\uMISC\\u            \\|\\uSCROLL\\u    \\|\\uSHELL\\u       \\|\\uGOTO\\u       \\|\\uI-SEARCH\\u     \\|\\i \\i\n"
-		"\\i \\i\\|^K[ 0-9 Record \\|^K SPACE status \\|^[W Up    \\|^K' Window  \\|^[B To ^KB \\|^[R Backwards\\|\\i \\i\n"
-		"\\i \\i\\|^K]     Stop   \\|^K\\\\ Repeat      \\|^[Z Down  \\|^[! Command \\|^[K To ^KK \\|^[S Forwards \\|\\i \\i\n"
-		"\\i \\i\\|^K 0-9  Play   \\|\\uINSERT\\u          \\|^K< Left  \\|\\uQUOTE\\u       \\|\\uDELETE\\u     \\|\\uBOOKMARKS\\u    \\|\\i \\i\n"
-		"\\i \\i\\|^K?     Query  \\|^] split line   \\|^K> Right \\|^[' Ctrl-   \\|^[Y yank   \\|^[ 0-9 Goto  \\|\\i \\i\n"
-		"\\i \\i\\|^[D     Dump   \\|^SPACE ins space\\|          \\|^\\\\ Meta-    \\|^[O line<  \\|^[^[   Set   \\|\\i \\i\n"
+		"\\|\\uMACROS\\u         \\|\\uSCROLL\\u           \\|\\uWINDOW\\u               \\|\\uGO TO\\u      \\|\\uI-SEARCH\\u     \\| \n"
+		"\\|\\b^K[ 0-9\\b Record \\|\\bEsc W\\b Up         \\|\\b^KO\\b Open  \\b^C\\b  Close  \\|\\bEsc B\\b / \\bK\\b  \\|\\b^R\\b Backwards \\| \n"
+		"\\|\\b^K]\\b     Stop   \\|\\bEsc Z\\b Down       \\|\\b^KG\\b Grow  \\b^KT\\b Shrink \\|to \\b^KB\\b/\\b^KK \\b\\|\\b^S\\b Forwards  \\| \n"
+		"\\|\\bEsc I\\b   Text   \\|\\bEsc <\\b Left       \\|\\b^KN\\b Next  \\b^KP\\b Prev.  \\|\\b^P\\b  Up     \\|\\uBOOKMARKS\\u    \\| \n"
+		"\\|\\b^K?\\b     Pause  \\|\\bEsc >\\b Right      \\|\\b^KI\\b Show one / All   \\|\\b^N\\b  Down   \\|\\bEsc 0-9\\b Goto \\| \n"
+		"\\|   for prompt  \\|\\uINSERT\\u           \\|\\uSHELL\\u                \\|\\b^B\\b  Left   \\|\\bEsc Esc\\b Set  \\| \n"
+		"\\|\\b^K 0-9\\b  Play   \\|\\b^]\\b Line  \\b^Q\\b Ctrl \\|\\bF1-F4\\b Term emulator  \\|\\b^F\\b  Right  \\|\\b^K-\\b Auto prev\\| \n"
+		"\\|\\b^K\\\\\\b     Repeat \\|\\b^@\\b Space \\b^\\\\\\b Meta \\|\\b^K'\\b In buffer        \\|           \\|\\b^K=\\b Auto next\\| \n"
+		"\\|\\bEsc D\\b   Dump   \\|\\bEsc Y\\b Paste      \\|\\b^KZ\\b Suspend  \\bEsc !\\b Append \\|      \\|             \\| \n"
 		"}\n"
 		"{Programs\n"
-		"\\i   Help Screen    \\|turn off with ^KH    prev. screen ^[,    next screen ^[.     \\i\n"
-		"\\i \\i\\|\\uGOTO\\u                 \\|\\uINDENT\\u   \\|\\uCOMPILING\\u          \\uQUICK\\u                       \\|\\i \\i\n"
-		"\\i \\i\\|^G  Matching ( [ {   \\|^K, less \\|^[C Compile & parse ^G  find word under cursor \\|\\i \\i\n"
-		"\\i \\i\\|^K- Previous place   \\|^K. more \\|^[G Grep & parse    ^[ENTER complete word      \\|\\i \\i\n"
-		"\\i \\i\\|^K= Next place       \\|\\uFILE\\u     \\|^[= To next error   ^[Y paste deleted text     \\|\\i \\i\n"
-		"\\i \\i\\|^K; Tags file search \\|^K `     \\|^[- To prev. error  ^[^Y cycle through deleted \\|\\i \\i\n"
-		"\\i \\i\\|                     \\| revert  \\|^[ SP Jump to error                            \\|\\i \\i\n"
+		"\\|\\uCOMMANDS\\u            \\|\\uCOMPILING\\u                  \\|\\uQUICK\\u                       \\| \n"
+		"\\|\\bEsc X\\b Command prompt\\|\\bEsc C\\b Save modified compile\\|\\b^G\\b  Find word under cursor  \\| \n"
+		"\\|\\bEsc M\\b Calculator    \\|\\bEsc G\\b Grep                 \\|\\b^K;\\b Tags file search        \\| \n"
+		"\\|\\bshowlog\\b Startup log \\|\\bEsc =\\b To next error        \\|\\bEsc Enter\\b Complete word     \\| \n"
+		"\\|\\bupper\\b   Change case \\|\\bEsc -\\b To prev. error       \\|\\uBUFFER\\u                      \\| \n"
+		"\\|\\blower\\b   Change case \\|\\bEsc Space\\b Jump to error    \\|\\bEsc U\\b Next                  \\| \n"
+		"\\|\\bcd\\b      Change dir  \\|\\uINDENT\\u                     \\|\\bEsc V\\b Previous              \\| \n"
+		"\\|\\bkeymap\\b  Change map  \\|\\b^K,\\b Less                   \\|\\bEsc X bufed\\b Switch          \\| \n"
+		"\\|                    \\|\\b^K.\\b More                   \\|                            \\| \n"
 		"}\n"
 		"{Search\n"
-		"\\i   Help Screen    \\|turn off with ^KH    prev. screen ^[,    next screen ^[.     \\i\n"
-		"\\i \\iSearch sequences:                                                            \\|\\i \\i\n"
-		"\\i \\i    \\\\^  \\\\$  matches beg./end of line       \\\\.     match any single char      \\|\\i \\i\n"
-		"\\i \\i    \\\\<  \\\\>  matches beg./end of word       \\\\!     match char or expression   \\|\\i \\i\n"
-		"\\i \\i    \\\\(  \\\\)  grouping                       \\\\|     match left or right         \\|\\i \\i\n"
-		"\\i \\i    \\\\[a-z]  matches one of a set                                             \\|\\i \\i\n"
-		"\\i \\i    \\\\{1,3}  match 1 - 3 occurrences        \\\\?     match 0 or 1 occurrence     \\|\\i \\i\n"
-		"\\i \\i    \\\\+      match 1 or more occurrences    \\\\*     match 0 or more occurrences \\|\\i \\i\n"
-		"\\i \\iReplace sequences:                                                           \\|\\i \\i\n"
-		"\\i \\i    \\\\&      replaced with entire match     \\\\1 - 9 replaced with Nth group   \\|\\i \\i\n"
-		"\\i \\i    \\\\u \\\\l   convert next to upper/lower    \\\\U \\\\L  case convert until \\\\E     \\|\\i \\i\n"
+		"Search sequences:                                                            \\| \n"
+		"    \\b\\\\^  \\\\$\\b  matches beg./end of line       \\b\\\\.\\b     match any single char      \\| \n"
+		"    \\b\\\\<  \\\\>\\b  matches beg./end of word       \\b\\\\!\\b     match char or expression   \\| \n"
+		"    \\b\\\\(  \\\\)\\b  grouping                       \\b\\\\|\\b     match left or right         \\| \n"
+		"    \\b\\\\[a-z]\\b  matches one of a set           \\b\\\\y \\\\Y\\b  short for (.*) / (!*)       \\| \n"
+		"    \\b\\\\{1,3}\\b  match 1 - 3 occurrences        \\b\\\\?\\b     match 0 or 1 occurrence     \\| \n"
+		"    \\b\\\\+\\b      match 1 or more occurrences    \\b\\\\*\\b     match 0 or more occurrences \\| \n"
+		"                                                                              \\|\n"
+		"\\b^C\\b    to abort                             \\bEsc .\\b  to list of escape sequences \\|\n"
 		"}\n"
 		"{Escape sequences\n"
-		"\\i   Help Screen    \\|turn off with ^KH    prev. screen ^[,    next screen ^[.     \\i\n"
-		"\\i \\iEscape sequences: \\\\x{10fff}  Unicode code point   \\\\p{Ll}  Unicode category \\|\\i \\i\n"
-		"\\i \\i    \\\\i / \\\\I  Identifier start      \\\\t  tab          \\\\e  escape               \\|\\i \\i\n"
-		"\\i \\i    \\\\c / \\\\C  Identifier continue   \\\\n  newline      \\\\r  carriage return      \\|\\i \\i\n"
-		"\\i \\i    \\\\d / \\\\D  Digit / Not a digit   \\\\b  backspace  \\\\xFF  hex character        \\|\\i \\i\n"
-		"\\i \\i    \\\\w / \\\\W  Word / Not a word     \\\\a  alert      \\\\377  octal character      \\|\\i \\i\n"
-		"\\i \\i    \\\\s / \\\\S  Space / Not a space   \\\\f  formfeed     \\\\\\\\  backslash            \\|\\i \\i\n"
+		"Escape sequences: \\| \n"
+		"   \\b\\\\x{10fff}\\b Unicode code point    \\b\\\\p{Ll}\\b  Unicode category \\| \n"
+		"   \\b\\\\i\\b / \\b\\\\I\\b   Identifier start      \\b\\\\t\\b  tab          \\b\\\\e\\b  escape               \\| \n"
+		"   \\b\\\\c\\b / \\b\\\\C\\b   Identifier continue   \\b\\\\n\\b  newline      \\b\\\\r\\b  carriage return      \\| \n"
+		"   \\b\\\\d\\b / \\b\\\\D\\b   Digit / Not a digit   \\b\\\\b\\b  backspace  \\b\\\\xFF\\b  hex character        \\| \n"
+		"   \\b\\\\w\\b / \\b\\\\W\\b   Word / Not a word     \\b\\\\a\\b  alert      \\b\\\\377\\b  octal character      \\| \n"
+		"   \\b\\\\s\\b / \\b\\\\S\\b   Space / Not a space   \\b\\\\f\\b  formfeed     \\b\\\\\\\\\\b  backslash            \\| \n"
+		" \\|\n"
+		" \\|\n"
+		"}\n"
+		"{Replace\n"
+		"Replace sequences:                                                           \\| \n"
+		"    \\b\\\\&\\b      replaced with entire match     \\b\\\\1\\b - \\b9\\b replaced with Nth group   \\| \n"
+		"    \\b\\\\u \\\\l\\b   convert next to upper/lower    \\b\\\\U \\\\L\\b  case convert until \\b\\\\E\\b     \\| \n"
+		"                                                                             \\|\n"
+		"                                                                             \\|\n"
+		"\\|\n"
+		"\\|\n"
+		"\\|\n"
+		"\\b^C\\b    to abort                             \\bEsc ,\\b to list of escape sequences \\|\n"
 		"}\n"
 		"{SearchOptions\n"
-		"\\i   Help Screen    \\|turn off with ^KH    prev. screen ^[,    next screen ^[.     \\i\n"
-		"\\i \\iSearch options:                                                              \\|\\i \\i\n"
-		"\\i \\i     r Replace      k Restrict search to highlighted block                   \\|\\i \\i\n"
-		"\\i \\i     i Ignore case  b Search backwards instead of forwards                   \\|\\i \\i\n"
-		"\\i \\i                    a Search across all loaded files                         \\|\\i \\i\n"
-		"\\i \\i                    e Search across all files in Grep or Compile error list  \\|\\i \\i\n"
-		"\\i \\i w / n  Allow / prevent wrap to start of file                                \\|\\i \\i\n"
-		"\\i \\i x / y  Search text is standard format / JOE format regular expression       \\|\\i \\i\n"
-		"\\i \\i   nnn  Perform exactly nnn replacements                                     \\|\\i \\i\n"
+		"Search options:                                                              \\| \n"
+		"     \\br\\b Replace      \\bk\\b Restrict search to highlighted block                   \\| \n"
+		"     \\bi\\b Ignore case  \\bb\\b Search backwards instead of forwards                   \\| \n"
+		"                    \\ba\\b Search across all loaded files                         \\| \n"
+		"                    \\be\\b Search across all files in Grep or Compile error list  \\| \n"
+		" \\bw\\b / \\bn\\b  Allow / prevent wrap to start of file                                \\| \n"
+		" \\bx\\b / \\by\\b  Search text is standard format / JOE format regular expression       \\| \n"
+		"   \\bnnn\\b  Perform exactly nnn replacements                                     \\| \n"
+		"\\b^C\\b    to abort\n"
 		"}\n"
 		"{Math\n"
-		"\\i   Help Screen    \\|turn off with ^KH    prev. screen ^[,    next screen ^[.     \\i\n"
-		"\\i \\i \\uCOMMANDS\\u (hit ESC m for math)  \\uFUNCTIONS\\u                                    \\|\\i \\i\n"
-		"\\i \\i     hex hex display mode       sin cos tab asin acos atan                   \\|\\i \\i\n"
-		"\\i \\i     dec decimal mode           sinh cosh tanh asinh acosh atanh             \\|\\i \\i\n"
-		"\\i \\i     ins type result into file  sqrt cbrt exp ln log                         \\|\\i \\i\n"
-		"\\i \\i    eval evaluate block         int floor ceil abs erg ergc                  \\|\\i \\i\n"
-		"\\i \\i    0xff enter number in hex    joe(..macro..) - runs an editor macro        \\|\\i \\i\n"
-		"\\i \\i    3e-4 floating point decimal \\uBLOCK\\u                                        \\|\\i \\i\n"
-		"\\i \\i    a=10 assign a variable      sum cnt  Sum, count                          \\|\\i \\i\n"
-		"\\i \\i 2+3:ins multiple commands      avg dev  Average, std. deviation             \\|\\i \\i\n"
-		"\\i \\i    e pi constants              \\uOPERATORS\\u                                    \\|\\i \\i\n"
-		"\\i \\i     ans previous result        ! ^  * / %  + -  < <= > >= == !=  &&  ||  ? :\\|\\i \\i\n"
+		" \\uMATH COMMANDS\\u                  \\uCONSTANTS\\u                                \\| \n"
+		" \\bhex oct bin\\b display mode       \\b0xff\\b hex entry                           \\| \n"
+		" \\beng dec\\b engineering / normal   \\b0o77\\b octal entry                         \\| \n"
+		"     \\bins\\b type result into file  \\b0b11\\b binary entry                        \\| \n"
+		"    \\beval\\b evaluate block         \\b3e-4\\b floating point entry                \\| \n"
+		" 2+3\\b:\\bins multiple commands      \\uVARIABLES\\u                                \\| \n"
+		" \\uOPERATORS\\u                      \\bans\\b  Previous result                     \\| \n"
+		" \\b! ^  * / %  + -\\b                \\ba=10\\b Assign a variable                   \\| \n"
+		" \\b< <= > >= == !=  &&  ||  ? :\\b   \\bEsc .\\b for list of functions              \\| \n"
+		"}\n"
+		"{Functions\n"
+		" \\uFUNCTIONS\\u                      \\uCONSTANTS\\u                                \\| \n"
+		" \\bsin cos tab asin acos atan\\b     \\be pi\\b                                     \\| \n"
+		" \\bsinh cosh tanh asinh acosh atanh\\b                                        \\| \n"
+		" \\bsqrt cbrt exp ln log\\b                                                    \\| \n"
+		" \\bint floor ceil abs\\b                                                      \\| \n"
+		" \\berf erfc j0 j1 y0 y1\\b                                                    \\| \n"
+		" \\bjoe(..macro..)\\b - runs an editor macro                                   \\| \n"
+		"                                                                             \\| \n"
+		"                                \\bEsc .\\b for statistics                     \\| \n"
+		"}\n"
+		"{Statistics \n"
+		" \\uMATH COMMANDS\\u                                                               \\| \n"
+		" Statistics on region of numbers  Linear regression on region of x y pairs   \\| \n"
+		" \\bsum\\b  Sum                         \\blr(x)\\b  Estimate y given x     y=m*x+b      \\| \n"
+		" \\bcnt\\b  Count                       \\brlr(y)\\b Estimate x given y                  \\| \n"
+		" \\bavg\\b  Mean                        \\bLr rLr\\b Logarithmic regression y=m*ln(x)+b  \\| \n"
+		"   Standard deviation:            \\blR rlR\\b Exponential regression y=e^(m*x)+b  \\| \n"
+		" \\bdev\\b  all data present            \\bLR rLR\\b Power regression       y=b*x^m      \\| \n"
+		" \\bsamp\\b sample of data present      Side effects: \\bb\\b, \\bm\\b: regression coef.       \\| \n"
+		"                                      \\br\\b: correlation coef., \\bcov\\b: covariance  \\| \n"
 		"}\n"
 		"{Names\n"
-		"\\i   Help Screen    \\|turn off with ^KH    prev. screen ^[,    next screen ^[.     \\i\n"
-		"\\i \\i Hit TAB at file name prompts to generate menu of file names                 \\|\\i \\i\n"
-		"\\i \\i Or use up/down keys to access history of previously entered names           \\|\\i \\i\n"
-		"\\i \\i Special file names:                                                         \\|\\i \\i\n"
-		"\\i \\i      !command                 Pipe in/out of a shell command                \\|\\i \\i\n"
-		"\\i \\i      >>filename               Append to a file                              \\|\\i \\i\n"
-		"\\i \\i      -                        Read/Write to/from standard I/O               \\|\\i \\i\n"
-		"\\i \\i      filename,START,SIZE      Read/Write a part of a file/device            \\|\\i \\i\n"
-		"\\i \\i          Give START/SIZE in decimal (255), octal (0377) or hex (0xFF)       \\|\\i \\i\n"
-		"}\n"
-		"{Joe\n"
-		"\\i   Help Screen    \\|turn off with ^KH    prev. screen ^[,    next screen ^[.     \\i\n"
-		"\\i \\i Send bug reports to: http://sourceforge.net/projects/joe-editor \\|\\i \\i\n"
-		"\\i \\i \\|\\i \\i\n"
-		"\\i \\i  default joerc file is here /home/jhallen/etc/joe/joerc \\|\\i \\i\n"
-		"\\i \\i  default syntax and i18n files are here /home/jhallen/share/joe \\|\\i \\i\n"
-		"\\i \\i  additional documentation can be found here /home/jhallen/share/doc/joe \\|\\i \\i\n"
+		" Hit TAB to complete names.  Use up / down arrow keys to traverse history.   \\| \n"
+		" Special file names:                                                         \\| \n"
+		"      \\b!\\bcommand                 Pipe in/out of a shell command                \\| \n"
+		"      \\b>>\\bfilename               Append to a file                              \\| \n"
+		"      \\b-\\b                        Read/Write to/from standard I/O               \\|\n"
+		"      filename\\b,START,SIZE\\b      Read/Write a part of a file/device            \\| \n"
+		"          Give START/SIZE in decimal (255), octal (0377) or hex (0xFF)       \\| \n"
+		"      \\b\\\\,\\b                       Use backslash to quote special characters     \\|\n"
+		"\\b^C\\b    to abort\n"
 		"}\n"
 		"{CharTable\n"
-		"\\i   Help Screen    \\|turn off with ^KH    prev. screen ^[,                        \\i\n"
-		"\\i \\i\\| Dec  \\u 0123 4567  8901 2345    0123 4567  8901 2345 \\u  Dec \\|\\i \\i\n"
-		"\\i \\i\\|     |                                              |     \\|\\i \\i\n"
-		"\\i \\i\\|   0 | \\u@ABC\\u \\uDEFG\\u  \\uHIJK\\u \\uLMNO\\u    \\i\\u@ABC\\u\\i \\i\\uDEFG\\u\\i  \\i\\uHIJK\\u\\i \\i\\uLMNO\\u\\i | 128 \\|\\i \\i\n"
-		"\\i \\i\\|  16 | \\uPQRS\\u \\uTUVW\\u  \\uXYZ[\\u \\u\\\\]^_\\u    \\i\\uPQRS\\u\\i \\i\\uTUVW\\u\\i  \\i\\uXYZ[\\u\\i \\i\\u\\\\]^_\\u\\i | 144 \\|\\i \\i\n"
-		"\\i \\i\\|  32 |  !\"# $%&'  ()*+ ,-./    \xa0\xa1\xa2\xa3 \xa4\xa5\xa6\xa7  \xa8\xa9\xaa\xab \xac\xad\xae\xaf | 160 \\|\\i \\i\n"
-		"\\i \\i\\|  48 | 0123 4567  89:; <=>?    \xb0\xb1\xb2\xb3 \xb4\xb5\xb6\xb7  \xb8\xb9\xba\xbb \xbc\xbd\xbe\xbf | 176 \\|\\i \\i\n"
-		"\\i \\i\\|  64 | @ABC DEFG  HIJK LMNO    \xc0\xc1\xc2\xc3 \xc4\xc5\xc6\xc7  \xc8\xc9\xca\xcb \xcc\xcd\xce\xcf | 192 \\|\\i \\i\n"
-		"\\i \\i\\|  80 | PQRS TUVW  XYZ[ \\\\]^_    \xd0\xd1\xd2\xd3 \xd4\xd5\xd6\xd7  \xd8\xd9\xda\xdb \xdc\xdd\xde\xdf | 208 \\|\\i \\i\n"
-		"\\i \\i\\|  96 | `abc defg  hijk lmno    \xe0\xe1\xe2\xe3 \xe4\xe5\xe6\xe7  \xe8\xe9\xea\xeb \xec\xed\xee\xef | 224 \\|\\i \\i\n"
-		"\\i \\i\\| 112 | pqrs tuvw  xyz{ |}~\x7f    \xf0\xf1\xf2\xf3 \xf4\xf5\xf6\xf7  \xf8\xf9\xfa\xfb \xfc\xfd\xfe\xff | 240 \\|\\i \\i\n"
+		"\\| Dec  \\u 0123 4567  8901 2345    0123 4567  8901 2345 \\u  Dec \\| \n"
+		"\\|   0 | \\u@ABC\\u \\uDEFG\\u  \\uHIJK\\u \\uLMNO\\u    \\i\\u@ABC\\u\\i \\i\\uDEFG\\u\\i  \\i\\uHIJK\\u\\i \\i\\uLMNO\\u\\i | 128 \\| \n"
+		"\\|  16 | \\uPQRS\\u \\uTUVW\\u  \\uXYZ[\\u \\u\\\\]^_\\u    \\i\\uPQRS\\u\\i \\i\\uTUVW\\u\\i  \\i\\uXYZ[\\u\\i \\i\\u\\\\]^_\\u\\i | 144 \\| \n"
+		"\\|  32 |  !\"# $%&'  ()*+ ,-./    \xa0\xa1\xa2\xa3 \xa4\xa5\xa6\xa7  \xa8\xa9\xaa\xab \xacX\xae\xaf | 160 \\| \n"
+		"\\|  48 | 0123 4567  89:; <=>?    \xb0\xb1\xb2\xb3 \xb4\xb5\xb6\xb7  \xb8\xb9\xba\xbb \xbc\xbd\xbe\xbf | 176 \\| \n"
+		"\\|  64 | @ABC DEFG  HIJK LMNO    \xc0\xc1\xc2\xc3 \xc4\xc5\xc6\xc7  \xc8\xc9\xca\xcb \xcc\xcd\xce\xcf | 192 \\| \n"
+		"\\|  80 | PQRS TUVW  XYZ[ \\\\]^_    \xd0\xd1\xd2\xd3 \xd4\xd5\xd6\xd7  \xd8\xd9\xda\xdb \xdc\xdd\xde\xdf | 208 \\| \n"
+		"\\|  96 | `abc defg  hijk lmno    \xe0\xe1\xe2\xe3 \xe4\xe5\xe6\xe7  \xe8\xe9\xea\xeb \xec\xed\xee\xef | 224 \\| \n"
+		"\\| 112 | pqrs tuvw  xyz{ |}~\x7f    \xf0\xf1\xf2\xf3 \xf4\xf5\xf6\xf7  \xf8\xf9\xfa\xfb \xfc\xfd\xfe\xff | 240 \\| \n"
 		"}\n"
+		"{Joe\n"
+		" Send bug reports to: http://sourceforge.net/projects/joe-editor \\| \n"
+		" \\| \n"
+		"  default joerc file is here /home/allenjo/etc/joe/joerc \\| \n"
+		"  default syntax and i18n files are here /home/allenjo/share/joe \\| \n"
+		"  additional documentation can be found here /home/allenjo/share/doc/joe \\| \n"
+		" \\| \n"
+		" \\| \n"
+		" \\| \n"
+		" \\| \n"
+		"}\n"
+		":def recenter retype,arg,\"top+height/2-line\",rtn,upslide\n"
 		":windows		Bindings common to all windows\n"
 		"type		U+0 TO U+10FFFF\n"
 		"abort		^C		Abort window\n"
-		"abort		^K Q\n"
-		"abort		^K ^Q\n"
-		"abort		^K q\n"
+		"querysave,query,killjoe	^K Q	Query to save files, then exit\n"
+		"querysave,query,killjoe	^K ^Q\n"
+		"querysave,query,killjoe	^K q\n"
 		"arg		^K \\		Repeat next command\n"
 		"explode		^K I		Show all windows or show only one window\n"
 		"explode		^K ^I\n"
@@ -284,7 +320,6 @@ const char *builtins[]=
 		"prevw		^K p\n"
 		"query		^K ?		Macro query insert\n"
 		"record		^K [		Record a macro\n"
-		"retype		^R		Refresh screen\n"
 		"rtn		^M		Return\n"
 		"shell		^K Z		Shell escape\n"
 		"shell		^K ^Z\n"
@@ -299,6 +334,8 @@ const char *builtins[]=
 		"defm3down	M3DOWN		Select the line at the mouse location\n"
 		"defm3up		M3UP\n"
 		"defm3drag	M3DRAG		Select a region of lines\n"
+		"defmiddleup	MIDDLEUP\n"
+		"defmiddledown	MIDDLEDOWN	Insert text\n"
 		"xtmouse		^[ [ M		Introduces an xterm mouse event\n"
 		"extmouse	^[ [ <		Introduces an extended xterm mouse event\n"
 		":main			Text editing window\n"
@@ -321,7 +358,8 @@ const char *builtins[]=
 		":def shell_gparse gparse\n"
 		":def shell_release release\n"
 		":def shell_math maths\n"
-		":def shell_typemath txt,math,\"ins\",rtn,rtn,txt,\"\x0d\x04\",rtn\n"
+		":def shell_abort abort\n"
+		":def shell_typemath txt,maths,\"ins\",rtn,rtn,txt,\"\x0d\",rtn,txt,quote,\"d\",rtn\n"
 		":def shell_rtn rtn\n"
 		":def shell_edit edit\n"
 		":def shell_dellin dellin\n"
@@ -382,6 +420,7 @@ const char *builtins[]=
 		"isrch		^[ s		Forward incremental search\n"
 		"isrch		^[ S		Forward incremental search\n"
 		"isrch		^[ ^S		Forward incremental search\n"
+		"isrch		^S\n"
 		"notmod		^[ ~		Not modified\n"
 		"mwind,mfit,prevw,nxterr		^[ =		To next error\n"
 		"parserr		^[ e		Parse errors in current buffer\n"
@@ -391,6 +430,7 @@ const char *builtins[]=
 		"rsrch		^[ r		Backward incremental search\n"
 		"rsrch		^[ R		Backward incremental search\n"
 		"rsrch		^[ ^R		Backward incremental search\n"
+		"retype,rsrch	^R\n"
 		"run		^[ !		Run a program in a window\n"
 		"tomarkb		^[ b		Go to beginning of marked block\n"
 		"tomarkb		^[ ^B		Go to beginning of marked block\n"
@@ -441,7 +481,9 @@ const char *builtins[]=
 		"center		^K ^A\n"
 		"center		^K a\n"
 		"crawll		^K <		Pan left\n"
+		"crawll		^[ <		Pan left\n"
 		"crawlr		^K >		Pan right\n"
+		"crawlr		^[ >		Pan right\n"
 		"delch		.kD		Delete character\n"
 		"delch		^D\n"
 		"deleol		^J		Delete to end of line\n"
@@ -706,12 +748,14 @@ const char *builtins[]=
 		"[mason]\n"
 		"*.mas\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax mason\n"
 		"-smarthome\n"
 		"-smartbacks\n"
 		"[sml]\n"
 		"*.sml\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax sml\n"
 		"-smarthome\n"
 		"-smartbacks\n"
@@ -720,6 +764,7 @@ const char *builtins[]=
 		"*.ml\n"
 		"*.mli\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax ocaml\n"
 		"-smarthome\n"
 		"-smartbacks\n"
@@ -734,6 +779,7 @@ const char *builtins[]=
 		"*Guardfile\n"
 		"*.rabl\n"
 		"-autoindent\n"
+		"-title\n"
 		"-pound_comment\n"
 		"-syntax ruby\n"
 		"-text_delimiters do=end\n"
@@ -744,6 +790,7 @@ const char *builtins[]=
 		"*.pm\n"
 		"*.t\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax perl\n"
 		"-smarthome\n"
 		"-smartbacks\n"
@@ -752,6 +799,7 @@ const char *builtins[]=
 		"[sql]\n"
 		"*.sql\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax sql\n"
 		"-smarthome\n"
 		"-smartbacks\n"
@@ -765,6 +813,7 @@ const char *builtins[]=
 		"[awk]\n"
 		"*.awk\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax awk\n"
 		"-smarthome\n"
 		"-smartbacks\n"
@@ -773,6 +822,7 @@ const char *builtins[]=
 		"[yacc]\n"
 		"*.y\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax c\n"
 		"-highlighter_context\n"
 		"-smarthome\n"
@@ -787,6 +837,7 @@ const char *builtins[]=
 		"*.l\n"
 		"*.lex\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax c\n"
 		"-highlighter_context\n"
 		"-smarthome\n"
@@ -802,6 +853,7 @@ const char *builtins[]=
 		"*.ads\n"
 		"-syntax ada\n"
 		"-autoindent\n"
+		"-title\n"
 		"-istep 2\n"
 		"-smarthome\n"
 		"-smartbacks\n"
@@ -819,6 +871,7 @@ const char *builtins[]=
 		"+#![ 	]*[^\\n]*rexx[0-9.]*\\>\n"
 		"-syntax rexx\n"
 		"-autoindent\n"
+		"-title\n"
 		"[sed]\n"
 		"*.sed\n"
 		"-syntax sed\n"
@@ -838,6 +891,7 @@ const char *builtins[]=
 		"*.hh\n"
 		"*.mm\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax c\n"
 		"-highlighter_context\n"
 		"-smarthome\n"
@@ -904,6 +958,7 @@ const char *builtins[]=
 		"[java]\n"
 		"*.java\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax java\n"
 		"-smarthome\n"
 		"-smartbacks\n"
@@ -916,6 +971,7 @@ const char *builtins[]=
 		"*\n"
 		"+#![ 	]*[^\\n]*nodejs[0-9.]*\\>\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax js\n"
 		"-smarthome\n"
 		"-smartbacks\n"
@@ -932,6 +988,7 @@ const char *builtins[]=
 		"[typescript]\n"
 		"*.ts\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax typescript\n"
 		"-smarthome\n"
 		"-smartbacks\n"
@@ -942,6 +999,7 @@ const char *builtins[]=
 		"[php]\n"
 		"*.php\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax php\n"
 		"-smarthome\n"
 		"-smartbacks\n"
@@ -951,6 +1009,7 @@ const char *builtins[]=
 		"+#![ 	]*[^\\n]*python[0-9.]*\\>\n"
 		"*.py\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax python\n"
 		"-highlighter_context\n"
 		"-smarthome\n"
@@ -968,6 +1027,7 @@ const char *builtins[]=
 		"*.tcsh\n"
 		"*.tcshrc\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax csh\n"
 		"-pound_comment\n"
 		"[sh]\n"
@@ -981,6 +1041,7 @@ const char *builtins[]=
 		"*.bash_login\n"
 		"*.bash_logout\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax sh\n"
 		"-highlighter_context\n"
 		"-pound_comment\n"
@@ -990,6 +1051,7 @@ const char *builtins[]=
 		"*.lsp\n"
 		"*.el\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax lisp\n"
 		"-semi_comment\n"
 		"[ksh]\n"
@@ -997,6 +1059,7 @@ const char *builtins[]=
 		"+#![ 	]*[^\\n 	]*/ksh\\>\n"
 		"*.ksh\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax sh\n"
 		"-highlighter_context\n"
 		"-pound_comment\n"
@@ -1005,6 +1068,7 @@ const char *builtins[]=
 		"*.p\n"
 		"*.pas\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax pascal\n"
 		"-smarthome\n"
 		"-smartbacks\n"
@@ -1016,12 +1080,14 @@ const char *builtins[]=
 		"*.for\n"
 		"*.FOR\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax fortran\n"
 		"[skill]\n"
 		"*.il\n"
 		"-semi_comment\n"
 		"-syntax skill\n"
 		"-autoindent\n"
+		"-title\n"
 		"-istep 3\n"
 		"-indentc 32\n"
 		"-smartbacks\n"
@@ -1032,14 +1098,17 @@ const char *builtins[]=
 		"*\n"
 		"+#![ 	]*[^\\n]*lua[0-9.]*\\>\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax lua\n"
 		"[tcl]\n"
 		"*.tcl\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax tcl\n"
 		"[go]\n"
 		"*.go\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax go\n"
 		"-smarthome\n"
 		"-smartbacks\n"
@@ -1051,6 +1120,7 @@ const char *builtins[]=
 		"[c#]\n"
 		"*.cs\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax csharp\n"
 		"-smarthome\n"
 		"-smartbacks\n"
@@ -1061,12 +1131,14 @@ const char *builtins[]=
 		"*.jsf\n"
 		"*.jsf.in\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax jsf\n"
 		"-smarthome\n"
 		"[m4]\n"
 		"*.ac\n"
 		"*.m4\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax m4\n"
 		"[matlab]\n"
 		"*.m\n"
@@ -1155,15 +1227,18 @@ const char *builtins[]=
 		"*.erlang\n"
 		"-syntax erlang\n"
 		"-autoindent\n"
+		"-title\n"
 		"[sieve]\n"
 		"*.sieve\n"
 		"-syntax sieve\n"
 		"-autoindent\n"
+		"-title\n"
 		"[puppet]\n"
 		"*.pp\n"
 		"-syntax puppet\n"
 		"-pound_comment\n"
 		"-autoindent\n"
+		"-title\n"
 		"[iptables]\n"
 		"*\n"
 		"+# Generated by iptables-save\n"
@@ -1172,6 +1247,7 @@ const char *builtins[]=
 		"-pound_comment\n"
 		"[powershell]\n"
 		"*.ps1\n"
+		"*.psm1\n"
 		"-syntax powershell\n"
 		"[git-commit]\n"
 		"*COMMIT_EDITMSG\n"
@@ -1180,12 +1256,16 @@ const char *builtins[]=
 		"*.ex\n"
 		"*.exs\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax elixir\n"
 		"-pound_comment\n"
 		"-text_delimiters do=end\n"
+		"-indentc 32\n"
+		"-istep 2\n"
 		"[r]\n"
 		"*.r\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax r\n"
 		"-pound_comment\n"
 		"-smarthome\n"
@@ -1198,6 +1278,7 @@ const char *builtins[]=
 		"*.gy\n"
 		"*.gsh\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax groovy\n"
 		"-c_comment\n"
 		"-cpp_comment\n"
@@ -1211,6 +1292,7 @@ const char *builtins[]=
 		"*.cljc\n"
 		"*.edn\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax clojure\n"
 		"-semi_comment\n"
 		"-smarthome\n"
@@ -1219,6 +1301,7 @@ const char *builtins[]=
 		"[rust]\n"
 		"*.rs\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax rust\n"
 		"-c_comment\n"
 		"-cpp_comment\n"
@@ -1229,6 +1312,7 @@ const char *builtins[]=
 		"[coffee]\n"
 		"*.coffee\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax coffee\n"
 		"-pound_comment\n"
 		"-smarthome\n"
@@ -1238,6 +1322,7 @@ const char *builtins[]=
 		"[scala]\n"
 		"*.scala\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax scala\n"
 		"-c_comment\n"
 		"-cpp_comment\n"
@@ -1248,6 +1333,7 @@ const char *builtins[]=
 		"[swift]\n"
 		"*.swift\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax swift\n"
 		"-c_comment\n"
 		"-cpp_comment\n"
@@ -1258,6 +1344,7 @@ const char *builtins[]=
 		"[d]\n"
 		"*.d\n"
 		"-autoindent\n"
+		"-title\n"
 		"-syntax d\n"
 		"-c_comment\n"
 		"-cpp_comment\n"
@@ -1265,6 +1352,16 @@ const char *builtins[]=
 		"-smartbacks\n"
 		"-purify\n"
 		"-single_quoted\n"
+		"[dockerfile]\n"
+		"*Dockerfile\n"
+		"-autoindent\n"
+		"-title\n"
+		"-syntax dockerfile\n"
+		"-pound_comment\n"
+		"[spec]\n"
+		"*.spec\n"
+		"-autoindent\n"
+		"-syntax spec\n"
 ,	"c.jsf",
 		"\n"
 		"\n"
