@@ -12,12 +12,13 @@
 #endif
 
 char *exmsg = NULL;		/* Message to display when exiting the editor */
+char *xmsg;			/* Message to display when starting the editor */
 int usexmouse=0;
 int xmouse=0;
 int nonotice;
 int noexmsg = 0;
 int pastehack;
-int help;
+int helpon;
 
 Screen *maint;			/* Main edit screen */
 
@@ -28,7 +29,7 @@ void dofollows(void)
 	W *w = maint->curwin;
 
 	do {
-		if (w->y != -1 && w->watom->follow && w->object)
+		if (w->y != -1 && w->h && w->watom->follow && w->object)
 			w->watom->follow(w);
 		w = (W *)(w->link.next);
 	} while (w != maint->curwin);
@@ -211,7 +212,7 @@ int edloop(int flg)
 		}
 
 		/* Restore modes */
-		if (maint->curwin->watom->what & TYPETW) {
+		if (!leave && maint->curwin->watom->what & TYPETW) {
 			bw = (BW *)maint->curwin->object;
 
 			if (auto_off) {
@@ -309,6 +310,7 @@ int main(int argc, char **real_argv, const char * const *envv)
 	int omid;
 	int backopt;
 	int c;
+	int filesonly;
 
 	joe_iswinit();
 	joe_locale();
@@ -402,7 +404,7 @@ int main(int argc, char **real_argv, const char * const *envv)
 		} else {
 			nope:
 			vsrm(t);
-			/* Try Joe's bad english */
+			/* Try Joe's bad English */
 			t = vsncpy(NULL, 0, sc(JOERC));
 			t = vsncpy(sv(t), sv(run));
 			t = vsncpy(sv(t), sc("rc"));
@@ -488,7 +490,18 @@ int main(int argc, char **real_argv, const char * const *envv)
 		idleout = 0;
 
 	for (c = 1; argv[c]; ++c) {
-		if (argv[c][0] == '-') {
+		if (!strcmp(argv[c], "-help") || !strcmp(argv[c], "--help")) {
+			printf("Joe's Own Editor v%s\n\n", VERSION);
+			printf("Usage: %s [global-options] [ [local-options] filename ]...\n\n", argv[0]);
+			printf("Global options:\n");
+			cmd_help(0);
+			printf("\nLocal options:\n");
+			printf("    %-23s Start cursor on specified line\n", "+nnn");
+			cmd_help(1);
+			return 0;
+		} else if (argv[c][0] == '-') {
+			if (argv[c][1] == '-' && !argv[c][2])
+				break;
 			if (argv[c][1])
 				switch (glopt(argv[c] + 1, argv[c + 1], NULL, 1)) {
 				case 0:
@@ -520,15 +533,19 @@ int main(int argc, char **real_argv, const char * const *envv)
 	 * local options afterwords */
 
 	/* orphan is not compatible with exemac()- macros need a window to exist */
-	for (c = 1, backopt = 0; argv[c]; ++c)
-		if (argv[c][0] == '+' && argv[c][1]>='0' && argv[c][1]<='9') {
+	for (c = 1, backopt = 0, filesonly = 0; argv[c]; ++c)
+		if (!filesonly && argv[c][0] == '+' && argv[c][1]>='0' && argv[c][1]<='9') {
 			if (!backopt)
 				backopt = c;
-		} else if (argv[c][0] == '-' && argv[c][1]) {
-			if (!backopt)
-				backopt = c;
-			if (glopt(argv[c] + 1, argv[c + 1], NULL, 0) == 2)
-				++c;
+		} else if (!filesonly && argv[c][0] == '-' && argv[c][1]) {
+			if (argv[c][1] == '-' && !argv[c][2])
+				filesonly = 1;
+			else {
+				if (!backopt)
+					backopt = c;
+				if (glopt(argv[c] + 1, argv[c + 1], NULL, 0) == 2)
+					++c;
+			}
 		} else {
 			B *b = bfind(argv[c]);
 			BW *bw = NULL;
@@ -572,12 +589,12 @@ int main(int argc, char **real_argv, const char * const *envv)
 								backopt += 2;
 							else
 								backopt += 1;
-							/* lazy_opts(bw->b, &bw->o); */
 						}
 					}
 				}
 				/* bw->b->o = bw->o; */
 				lazy_opts(bw->b, &bw->o);
+				bw->o = bw->b->o;
 				bw->b->rdonly = bw->o.readonly;
 				/* Put cursor in window, so macros work properly */
 				maint->curwin = bw->parent;
@@ -626,13 +643,18 @@ int main(int argc, char **real_argv, const char * const *envv)
 		wshowall(maint);
 	}
 
-	if (help) {
+	if (helpon) {
 		help_on(maint);
 	}
 	if (!nonotice) {
-		joe_snprintf_3(msgbuf,JOE_MSGBUFSIZE,joe_gettext(_("\\i** Joe's Own Editor v%s ** (%s) ** Copyright %s 2015 **\\i")),VERSION,locale_map->name,(locale_map->type ? "©" : "(C)"));
+		if (xmsg) {
+			xmsg = stagen(NULL, (BW *)(lastw(maint)->object), joe_gettext(xmsg), ' ');
+			msgnw(((BASE *)lastw(maint)->object)->parent, xmsg);
+		} else {
+			joe_snprintf_3(msgbuf,JOE_MSGBUFSIZE,joe_gettext(_("\\i** Joe's Own Editor v%s ** (%s) ** Copyright %s 2015 **\\i")),VERSION,locale_map->name,(locale_map->type ? "©" : "(C)"));
+			msgnw(((BASE *)lastw(maint)->object)->parent, msgbuf);
+		}
 
-		msgnw(((BASE *)lastw(maint)->object)->parent, msgbuf);
 	}
 
 	if (!idleout) {
