@@ -21,7 +21,7 @@ int attr_size = 0;
 int stack_count = 0;
 static int state_count = 0; /* Max transitions possible without cycling */
 
-struct high_syntax ansi_syntax[1] = { { NULL, USTR "ansi" } };
+struct high_syntax *ansi_syntax;
 
 /* ANSI highlighter */
 
@@ -30,33 +30,29 @@ struct high_syntax ansi_syntax[1] = { { NULL, USTR "ansi" } };
 #define AFTER_BRACK 2
 #define IN_NUMBER 3
 
-HIGHLIGHT_STATE ansi_parse(P *line, HIGHLIGHT_STATE h_state)
+static HIGHLIGHT_STATE ansi_parse(P *line, HIGHLIGHT_STATE h_state)
 {
 	int *attr = attr_buf;
 	int *attr_end = attr_buf + attr_size;
 	int c;
 
-	int state = h_state.saved_s[0];
-	int accu = h_state.saved_s[1];
-	int current_attr = h_state.state;
+	int state = IDLE; /* h_state.saved_s[0]; */
+	int accu = 0; /* h_state.saved_s[1]; */
+	int current_attr = 0; /* (int)h_state.state; */ /* Do not let attributes cross lines - simplifies vt.c */
 	// int new_attr = *(int *)(h_state.saved_s + 8);
 
 	int ansi_mode = line->b->o.ansi;
 
-	current_attr = 0; /* Do not let attributes cross lines - simplifies vt.c */
-
 	line->b->o.ansi = 0;
 
 	while ((c = pgetc(line)) != NO_MORE_DATA) {
-		if (c < 0 || c > 255)
-			c = 0x1F;
 		if (attr == attr_end) {
 			if (!attr_buf) {
 				attr_size = 1024;
-				attr_buf = joe_malloc(sizeof(int) * attr_size);
+				attr_buf = (int *)joe_malloc(SIZEOF(int) * attr_size);
 				attr = attr_buf;
 			} else {
-				attr_buf = joe_realloc(attr_buf, sizeof(int) * (attr_size * 2));
+				attr_buf = (int *)joe_realloc(attr_buf, SIZEOF(int) * (attr_size * 2));
 				attr = attr_buf + attr_size;
 				attr_size *= 2;
 			}
@@ -83,7 +79,7 @@ HIGHLIGHT_STATE ansi_parse(P *line, HIGHLIGHT_STATE h_state)
 					current_attr = 0;
 					/* but stay in this state */
 				} else if (c >= '0' && c <= '9') {
-					accu = c - '0';
+					accu = (char)(c - '0');
 					state = IN_NUMBER;
 				} else if (c == 'm') {
 					/* APPLY NEW ATTRIBUTES */
@@ -117,7 +113,7 @@ HIGHLIGHT_STATE ansi_parse(P *line, HIGHLIGHT_STATE h_state)
 						state = IDLE;
 					}
 				} else if (c >= '0' && c <= '9') {
-					accu = accu * 10 + c - '0';
+					accu = (char)(accu * 10 + c - '0');
 				} else {
 					state = IDLE;
 				}
@@ -127,21 +123,21 @@ HIGHLIGHT_STATE ansi_parse(P *line, HIGHLIGHT_STATE h_state)
 			break;
 	}
 	line->b->o.ansi = ansi_mode;
-	h_state.saved_s[0] = state;
+	/* h_state.saved_s[0] = state;
 	h_state.saved_s[1] = accu;
+	h_state.saved_s[2] = 0; */ /* Because we Zcmp(saved_s) */
 	h_state.state = current_attr;
-//	*(int *)(h_state.saved_s + 8) = new_attr;
 	return h_state;
 }
 
-HIGHLIGHT_STATE parse(struct high_syntax *syntax,P *line,HIGHLIGHT_STATE h_state)
+HIGHLIGHT_STATE parse(struct high_syntax *syntax,P *line,HIGHLIGHT_STATE h_state,struct charmap *charmap)
 {
 	struct high_frame *stack;
 	struct high_state *h;
 			/* Current state */
-	unsigned char buf[24];		/* Name buffer (trunc after 23 characters) */
-	unsigned char lbuf[24];		/* Lower case version of name buffer */
-	unsigned char lsaved_s[24];	/* Lower case version of delimiter match buffer */
+	int buf[SAVED_SIZE];		/* Name buffer (trunc after 23 characters) */
+	int lbuf[3*SAVED_SIZE];		/* Lower case version of name buffer */
+	int lsaved_s[3*SAVED_SIZE];	/* Lower case version of delimiter match buffer */
 	int buf_idx;	/* Index into buffer */
 	int c;		/* Current character */
 	int *attr;
@@ -179,20 +175,20 @@ HIGHLIGHT_STATE parse(struct high_syntax *syntax,P *line,HIGHLIGHT_STATE h_state
 	while((c=pgetc(line))!=NO_MORE_DATA) {
 		struct high_cmd *cmd, *kw_cmd;
 		int iters = -8; /* +8 extra iterations before cycle detect. */
-		int x;
+		ptrdiff_t x;
 
-		/* Hack so we can have UTF-8 characters without crashing */
-		if (c < 0 || c > 255)
-			c = 0x1F;
+		/* If it isn't already, convert character to unicode */
+		if (!charmap->type)
+			c = to_uni(charmap, c);
 
 		/* Create or expand attribute array if necessary */
 		if(attr==attr_end) {
 			if(!attr_buf) {
 				attr_size = 1024;
-				attr_buf = joe_malloc(sizeof(int)*attr_size);
+				attr_buf = (int *)joe_malloc(SIZEOF(int)*attr_size);
 				attr = attr_buf;
 			} else {
-				attr_buf = joe_realloc(attr_buf,sizeof(int)*(attr_size*2));
+				attr_buf = (int *)joe_realloc(attr_buf,SIZEOF(int)*(attr_size*2));
 				attr = attr_buf + attr_size;
 				attr_size *= 2;
 			}
@@ -214,27 +210,33 @@ HIGHLIGHT_STATE parse(struct high_syntax *syntax,P *line,HIGHLIGHT_STATE h_state
 			attr[-1] = h->color;
 
 			/* Get command for this character */
-			if (h->delim && c == h_state.saved_s[0] && h_state.saved_s[1] == 0)
+			if (h->delim && h_state.saved_s && c == h_state.saved_s[0] && h_state.saved_s[1] && h_state.saved_s[2] == 0)
 				cmd = h->delim;
-			else
-				cmd = h->cmd[c];
+			else if (h->same_delim && h_state.saved_s && h_state.saved_s[0] && c == h_state.saved_s[1] && h_state.saved_s[2] == 0)
+				cmd = h->same_delim;
+			else {
+				cmd = (struct high_cmd *)rtree_lookup(&h->rtree, c);
+				if (!cmd)
+					cmd = h->dflt;
+			}
 
 			/* Lowerize strings for case-insensitive matching */
 			if (cmd->ignore) {
-				zlcpy(lbuf, sizeof(lbuf), buf);
-				lowerize(lbuf);
+				lowerize(lbuf, SIZEOF(lbuf)/SIZEOF(lbuf[0]), buf);
 				if (cmd->delim) {
-					zlcpy(lsaved_s, sizeof(lsaved_s), h_state.saved_s);
-					lowerize(lsaved_s);
+					if (h_state.saved_s)
+						lowerize(lsaved_s, SIZEOF(lsaved_s)/SIZEOF(lsaved_s[0]), h_state.saved_s);
+					else
+						lsaved_s[0] = 0;
 				}
 			}
 
 			/* Check for delimiter or keyword matches */
 			recolor_delimiter_or_keyword = 0;
-			if (cmd->delim && (cmd->ignore ? !zcmp(lsaved_s,lbuf) : !zcmp(h_state.saved_s,buf))) {
+			if (cmd->delim && (cmd->ignore ? !Zcmp(lsaved_s,lbuf) : (h_state.saved_s && !Zcmp(h_state.saved_s,buf)))) {
 				cmd = cmd->delim;
 				recolor_delimiter_or_keyword = 1;
-			} else if (cmd->keywords && (cmd->ignore ? (kw_cmd=htfind(cmd->keywords,lbuf)) : (kw_cmd=htfind(cmd->keywords,buf)))) {
+			} else if (cmd->keywords && (cmd->ignore ? (kw_cmd=(struct high_cmd *)Zhtfind(cmd->keywords,lbuf)) : (kw_cmd=(struct high_cmd *)Zhtfind(cmd->keywords,buf)))) {
 				cmd = kw_cmd;
 				recolor_delimiter_or_keyword = 1;
 			}
@@ -249,7 +251,7 @@ HIGHLIGHT_STATE parse(struct high_syntax *syntax,P *line,HIGHLIGHT_STATE h_state
 				if (*frame_ptr)
 					stack = *frame_ptr;
 				else {
-					struct high_frame *frame = joe_malloc(sizeof(struct high_frame));
+					struct high_frame *frame = (struct high_frame *)joe_malloc(SIZEOF(struct high_frame));
 					frame->parent = stack;
 					frame->child = 0;
 					frame->sibling = 0;
@@ -291,24 +293,28 @@ HIGHLIGHT_STATE parse(struct high_syntax *syntax,P *line,HIGHLIGHT_STATE h_state
 					attr[x] = h->color;
 
 			/* Save string? */
-			if (cmd->save_s)
-				zlcpy(h_state.saved_s, sizeof(h_state.saved_s), buf);
+			if (cmd->save_s) {
+				h_state.saved_s = Zatom_add(buf);
+			}
 
 			/* Save character? */
 			if (cmd->save_c) {
-				h_state.saved_s[1] = 0;
+				int bf[3];
+				bf[1] = c;
+				bf[2] = 0;
 				if (c=='<')
-					h_state.saved_s[0] = '>';
+					bf[0] = '>';
 				else if (c=='(')
-					h_state.saved_s[0] = ')';
+					bf[0] = ')';
 				else if (c=='[')
-					h_state.saved_s[0] = ']';
+					bf[0] = ']';
 				else if (c=='{')
-					h_state.saved_s[0] = '}';
+					bf[0] = '}';
 				else if (c=='`')
-					h_state.saved_s[0] = '\'';
+					bf[0] = '\'';
 				else
-					h_state.saved_s[0] = c;
+					bf[0] = c;
+				h_state.saved_s = Zatom_add(bf);
 			}
 
 			/* Start buffering? */
@@ -339,8 +345,8 @@ HIGHLIGHT_STATE parse(struct high_syntax *syntax,P *line,HIGHLIGHT_STATE h_state
 		} while(cmd->noeat);
 
 		/* Save character in buffer */
-		if (buf_idx<23 && buf_en)
-			buf[buf_idx++]=c;
+		if (buf_idx < (SAVED_SIZE - 1) && buf_en)
+			buf[buf_idx++] = c;
 		if (!buf_en)
 			++ofst;
 		buf[buf_idx] = 0;
@@ -361,33 +367,47 @@ HIGHLIGHT_STATE parse(struct high_syntax *syntax,P *line,HIGHLIGHT_STATE h_state
 
 /* Subroutines for load_dfa() */
 
-static struct high_state *find_state(struct high_syntax *syntax,unsigned char *name)
+static struct high_state *find_state(struct high_syntax *syntax,char *name)
 {
 	struct high_state *state;
 
 	/* Find state */
-	state = htfind(syntax->ht_states, name);
+	state = (struct high_state *)htfind(syntax->ht_states, name);
 
 	/* It doesn't exist, so create it */
 	if(!state) {
-		int y;
-		state=joe_malloc(sizeof(struct high_state));
+		state=(struct high_state *)joe_malloc(SIZEOF(struct high_state));
 		state->name=zdup(name);
 		state->no=syntax->nstates;
 		state->color=FG_WHITE;
 		/* Expand the state table if necessary */
 		if(syntax->nstates==syntax->szstates)
-			syntax->states=joe_realloc(syntax->states,sizeof(struct high_state *)*(syntax->szstates*=2));
+			syntax->states=(struct high_state **)joe_realloc(syntax->states,SIZEOF(struct high_state *)*(syntax->szstates*=2));
 		syntax->states[syntax->nstates++]=state;
-		for(y=0; y!=256; ++y)
-			state->cmd[y] = &syntax->default_cmd;
+		rtree_init(&state->rtree);
+		state->dflt = &syntax->default_cmd;
 		state->delim = 0;
+		state->same_delim = 0;
 		htadd(syntax->ht_states, state->name, state);
 		++state_count;
 	}
 	return state;
 }
 
+/* Build cmaps */
+
+static void build_cmaps(struct high_syntax *syntax)
+{
+	int x;
+	for (x = 0; x != syntax->ht_states->len; ++x) {
+		HENTRY *p;
+		for (p = syntax->ht_states->tab[x]; p; p = p->next) {
+			struct high_state *st = (struct high_state *)p->val;
+			rtree_opt(&st->rtree);
+		}
+	}
+}
+
 /* Create empty command */
 
 static void iz_cmd(struct high_cmd *cmd)
@@ -412,7 +432,7 @@ static void iz_cmd(struct high_cmd *cmd)
 
 static struct high_cmd *mkcmd()
 {
-	struct high_cmd *cmd = joe_malloc(sizeof(struct high_cmd));
+	struct high_cmd *cmd = (struct high_cmd *)joe_malloc(SIZEOF(struct high_cmd));
 	iz_cmd(cmd);
 	return cmd;
 }
@@ -421,11 +441,11 @@ static struct high_cmd *mkcmd()
 
 struct high_color *global_colors;
 
-struct high_color *find_color(struct high_color *colors,unsigned char *name,unsigned char *syn)
+static struct high_color *find_color(struct high_color *colors,char *name,char *syn)
 {
-	unsigned char bf[256];
+	char bf[256];
 	struct high_color *color;
-	joe_snprintf_2(bf, sizeof(bf), "%s.%s", syn, name);
+	joe_snprintf_2(bf, SIZEOF(bf), "%s.%s", syn, name);
 	for (color = colors; color; color = color->next)
 		if (!zcmp(color->name,bf)) break;
 	if (color)
@@ -435,9 +455,9 @@ struct high_color *find_color(struct high_color *colors,unsigned char *name,unsi
 	return color;
 }
 
-void parse_color_def(struct high_color **color_list,unsigned char *p,unsigned char *name,int line)
+void parse_color_def(struct high_color **color_list,const char *p,char *name,int line)
 {
-	unsigned char bf[256];
+	char bf[256];
 	if(!parse_tows(&p, bf)) {
 		struct high_color *color, *gcolor;
 
@@ -446,13 +466,13 @@ void parse_color_def(struct high_color **color_list,unsigned char *p,unsigned ch
 
 		/* If it doesn't exist, create it */
 		if(!color) {
-			color = joe_malloc(sizeof(struct high_color));
+			color = (struct high_color *)joe_malloc(SIZEOF(struct high_color));
 			color->name = zdup(bf);
 			color->color = 0;
 			color->next = *color_list;
 			*color_list = color;
 		} else {
-			logerror_2((char *)joe_gettext(_("%s %d: Class already defined\n")),name,line);
+			logerror_2(joe_gettext(_("%s %d: Class already defined\n")),name,line);
 		}
 
 		/* Find it in global list */
@@ -460,12 +480,12 @@ void parse_color_def(struct high_color **color_list,unsigned char *p,unsigned ch
 			color->color = gcolor->color;
 		} else {
 			/* Parse color definition */
-			while(parse_ws(&p,'#'), !parse_ident(&p,bf,sizeof(bf))) {
+			while(parse_ws(&p,'#'), !parse_ident(&p,bf,SIZEOF(bf))) {
 				color->color |= meta_color(bf);
 			}
 		}
 	} else {
-		logerror_2((char *)joe_gettext(_("%s %d: Missing class name\n")),name,line);
+		logerror_2(joe_gettext(_("%s %d: Missing class name\n")),name,line);
 	}
 }
 
@@ -479,58 +499,52 @@ void dump_syntax(BW *bw)
 {
 	struct high_syntax *syntax;
 	struct high_param *params;
-	unsigned char buf[1024];
-	joe_snprintf_1(buf, sizeof(buf), "Allocated %d stack frames\n", stack_count);
+	char buf[1024];
+	joe_snprintf_1(buf, SIZEOF(buf), "Allocated %d stack frames\n", stack_count);
 	binss(bw->cursor, buf);
 	pnextl(bw->cursor);
 	for (syntax = syntax_list; syntax; syntax = syntax->next) {
 		int x;
-		joe_snprintf_3(buf, sizeof(buf), "Syntax name=%s, subr=%s, nstates=%d\n",syntax->name,syntax->subr,syntax->nstates);
+		joe_snprintf_3(buf, SIZEOF(buf), "Syntax name=%s, subr=%s, nstates=%d\n",syntax->name,syntax->subr,(int)syntax->nstates);
 		binss(bw->cursor, buf);
 		pnextl(bw->cursor);
-		zlcpy(buf, sizeof(buf), USTR "params=(");
+		zlcpy(buf, SIZEOF(buf), "params=(");
 		for(params = syntax->params; params; params = params->next) {
-			zlcat(buf, sizeof(buf), USTR " ");
-			zlcat(buf, sizeof(buf), params->name);
+			zlcat(buf, SIZEOF(buf), " ");
+			zlcat(buf, SIZEOF(buf), params->name);
 		}
-		zlcat(buf, sizeof(buf), USTR " )\n");
+		zlcat(buf, SIZEOF(buf), " )\n");
 		binss(bw->cursor, buf);
 		pnextl(bw->cursor);
 		for(x=0;x!=syntax->nstates;++x) {
-			int y;
-			int f = -1;
 			struct high_state *s = syntax->states[x];
-			joe_snprintf_2(buf, sizeof(buf), "   state %s %x\n",s->name,s->color);
+			joe_snprintf_2(buf, SIZEOF(buf), "   state %s %x\n",s->name,s->color);
 			binss(bw->cursor, buf);
 			pnextl(bw->cursor);
-			for (y = 0; y != 256; ++y) {
-				if (f == -1)
-					f = y;
-				else if (s->cmd[f]->new_state != s->cmd[y]->new_state) {
-					joe_snprintf_4(buf, sizeof(buf), "     [%d-%d] -> %s %d\n",f,y-1,(s->cmd[f]->new_state ? s->cmd[f]->new_state->name : USTR "ERROR! Unknown state!"),s->cmd[f]->recolor);
-					binss(bw->cursor, buf);
-					pnextl(bw->cursor);
-					f = y;
-				}
-			}
-			joe_snprintf_4(buf, sizeof(buf), "     [%d-%d] -> %s %d\n",f,y-1,(s->cmd[f]->new_state ? s->cmd[f]->new_state->name : USTR "ERROR! Unknown state!"),s->cmd[f]->recolor);
+//			for (l = s->src; l; l = l->next) {
+//				struct high_cmd *h = (struct high_cmd *)l->map;
+//				joe_snprintf_4(buf, SIZEOF(buf), "     [%d-%d] -> %s %d\n",l->interval.first,l->interval.last,(h->new_state ? h->new_state->name : "ERROR! Unknown state!"),(int)h->recolor);
+//				binss(bw->cursor, buf);
+//				pnextl(bw->cursor);
+//			}
+			joe_snprintf_2(buf, SIZEOF(buf), "     default -> %s %d\n",(s->dflt->new_state ? s->dflt->new_state->name : "ERROR! Unknown state!"),(int)s->dflt->recolor);
 			binss(bw->cursor, buf);
 			pnextl(bw->cursor);
 		}
 	}
 }
 
-struct high_param *parse_params(struct high_param *current_params,unsigned char **ptr,unsigned char *name,int line)
+static struct high_param *parse_params(struct high_param *current_params,const char **ptr,char *name,int line)
 {
-	unsigned char *p = *ptr;
-	unsigned char bf[256];
+	const char *p = *ptr;
+	char bf[256];
 	struct high_param *params;
 	struct high_param **param_ptr;
 
 	/* Propagate currently defined parameters */
 	param_ptr = &params;
 	while (current_params) {
-		*param_ptr = joe_malloc(sizeof(struct high_param));
+		*param_ptr = (struct high_param *)joe_malloc(SIZEOF(struct high_param));
 		(*param_ptr)->name = zdup(current_params->name);
 		param_ptr = &(*param_ptr)->next;
 		current_params = current_params->next;
@@ -544,7 +558,7 @@ struct high_param *parse_params(struct high_param *current_params,unsigned char
 			if (!parse_char(&p, ')'))
 				break;
 			else if (!parse_char(&p, '-')) {
-				if (!parse_ident(&p,bf,sizeof(bf))) {
+				if (!parse_ident(&p,bf,SIZEOF(bf))) {
 					int cmp = 0;
 					param_ptr = &params;
 					/* Parameters are sorted */
@@ -557,9 +571,9 @@ struct high_param *parse_params(struct high_param *current_params,unsigned char
 						joe_free(param);
 					}
 				} else {
-					logerror_2((char *)joe_gettext(_("%s %d: Missing parameter name\n")),name,line);
+					logerror_2(joe_gettext(_("%s %d: Missing parameter name\n")),name,line);
 				}
-			} else if (!parse_ident(&p,bf,sizeof(bf))) {
+			} else if (!parse_ident(&p,bf,SIZEOF(bf))) {
 				int cmp = 0;
 				param_ptr = &params;
 				/* Keep parameters sorted */
@@ -567,13 +581,13 @@ struct high_param *parse_params(struct high_param *current_params,unsigned char
 					param_ptr = &(*param_ptr)->next;
 				/* Discard duplicates */
 				if (!*param_ptr || cmp) {
-					struct high_param *param = joe_malloc(sizeof(struct high_param));
+					struct high_param *param = (struct high_param *)joe_malloc(SIZEOF(struct high_param));
 					param->name = zdup(bf);
 					param->next = *param_ptr;
 					*param_ptr = param;
 				}
 			} else {
-				logerror_2((char *)joe_gettext(_("%s %d: Missing )\n")),name,line);
+				logerror_2(joe_gettext(_("%s %d: Missing )\n")),name,line);
 				break;
 			}
 		}
@@ -584,99 +598,101 @@ struct high_param *parse_params(struct high_param *current_params,unsigned char
 }
 
 
-struct high_syntax *load_syntax_subr(unsigned char *name,unsigned char *subr,struct high_param *params);
+struct high_syntax *load_syntax_subr(const char *name,char *subr,struct high_param *params);
 
 /* Parse options */
 
-int parse_options(struct high_syntax *syntax,struct high_cmd *cmd,JFILE *f,unsigned char *p,int parsing_strings,unsigned char *name,int line)
+static int parse_options(struct high_syntax *syntax,struct high_cmd *cmd,JFILE *f,const char *p,int parsing_strings,char *name,int line)
 {
-	unsigned char buf[1024];
-	unsigned char bf[256];
-	unsigned char bf1[256];
+	char buf[1024];
+	char bf[256];
+	char bf1[256];
 
-	while (parse_ws(&p,'#'), !parse_ident(&p,bf,sizeof(bf)))
-		if(!zcmp(bf,USTR "buffer")) {
+	while (parse_ws(&p,'#'), !parse_ident(&p,bf,SIZEOF(bf)))
+		if(!zcmp(bf,"buffer")) {
 			cmd->start_buffering = 1;
-		} else if(!zcmp(bf,USTR "hold")) {
+		} else if(!zcmp(bf,"hold")) {
 			cmd->stop_buffering = 1;
-		} else if(!zcmp(bf,USTR "save_c")) {
+		} else if(!zcmp(bf,"save_c")) {
 			cmd->save_c = 1;
-		} else if(!zcmp(bf,USTR "save_s")) {
+		} else if(!zcmp(bf,"save_s")) {
 			cmd->save_s = 1;
-		} else if(!zcmp(bf,USTR "recolor")) {
+		} else if(!zcmp(bf,"recolor")) {
 			parse_ws(&p,'#');
 			if(!parse_char(&p,'=')) {
 				parse_ws(&p,'#');
-				if(parse_int(&p,&cmd->recolor))
-					logerror_2((char *)joe_gettext(_("%s %d: Missing value for option\n")),name,line);
+				if(parse_diff(&p,&cmd->recolor))
+					logerror_2(joe_gettext(_("%s %d: Missing value for option\n")),name,line);
 			} else
-				logerror_2((char *)joe_gettext(_("%s %d: Missing value for option\n")),name,line);
-		} else if(!zcmp(bf,USTR "call")) {
+				logerror_2(joe_gettext(_("%s %d: Missing value for option\n")),name,line);
+		} else if(!zcmp(bf,"call")) {
 			parse_ws(&p,'#');
 			if(!parse_char(&p,'=')) {
 				parse_ws(&p,'#');
 				if (!parse_char(&p,'.')) {
-					zlcpy(bf, sizeof(bf), syntax->name);
+					zlcpy(bf, SIZEOF(bf), syntax->name);
 					goto subr;
-				} else if (parse_ident(&p,bf,sizeof(bf)))
-					logerror_2((char *)joe_gettext(_("%s %d: Missing value for option\n")),name,line);
+				} else if (parse_ident(&p,bf,SIZEOF(bf)))
+					logerror_2(joe_gettext(_("%s %d: Missing value for option\n")),name,line);
 				else {
 					if (!parse_char(&p,'.')) {
 						subr:
-						if (parse_ident(&p,bf1,sizeof(bf1)))
-							logerror_2((char *)joe_gettext(_("%s %d: Missing subroutine name\n")),name,line);
+						if (parse_ident(&p,bf1,SIZEOF(bf1)))
+							logerror_2(joe_gettext(_("%s %d: Missing subroutine name\n")),name,line);
 						cmd->call = load_syntax_subr(bf,bf1,parse_params(syntax->params,&p,name,line));
 					} else
 						cmd->call = load_syntax_subr(bf,0,parse_params(syntax->params,&p,name,line));
 				}
 			} else
-				logerror_2((char *)joe_gettext(_("%s %d: Missing value for option\n")),name,line);
-		} else if(!zcmp(bf,USTR "return")) {
+				logerror_2(joe_gettext(_("%s %d: Missing value for option\n")),name,line);
+		} else if(!zcmp(bf,"return")) {
 			cmd->rtn = 1;
-		} else if(!zcmp(bf,USTR "reset")) {
+		} else if(!zcmp(bf,"reset")) {
 			cmd->reset = 1;
-		} else if(!parsing_strings && (!zcmp(bf,USTR "strings") || !zcmp(bf,USTR "istrings"))) {
+		} else if(!parsing_strings && (!zcmp(bf,"strings") || !zcmp(bf,"istrings"))) {
 			if (bf[0]=='i')
 				cmd->ignore = 1;
-			while(jfgets(buf,1023,f)) {
+			while(jfgets(buf,sizeof(buf),f)) {
 				++line;
 				p = buf;
 				parse_ws(&p,'#');
 				if (*p) {
-					if(!parse_field(&p,USTR "done"))
+					int kwbuf[256];
+					int lkwbuf[256 * 3];
+					if(!parse_field(&p,"done"))
 						break;
-					if(parse_string(&p,bf,sizeof(bf)) >= 0) {
+					if(parse_Zstring(&p,kwbuf,SIZEOF(kwbuf)/SIZEOF(kwbuf[0])) >= 0) {
 						parse_ws(&p,'#');
 						if (cmd->ignore)
-							lowerize(bf);
-						if(!parse_ident(&p,bf1,sizeof(bf1))) {
+							lowerize(lkwbuf, SIZEOF(lkwbuf)/SIZEOF(lkwbuf[0]), kwbuf);
+						if(!parse_ident(&p,bf1,SIZEOF(bf1))) {
 							struct high_cmd *kw_cmd=mkcmd();
 							kw_cmd->noeat=1;
 							kw_cmd->new_state = find_state(syntax,bf1);
-							if (!zcmp(bf, USTR "&")) {
+							if (kwbuf[0] == '&' && !kwbuf[1]) {
 								cmd->delim = kw_cmd;
 							} else {
 								if(!cmd->keywords)
-									cmd->keywords = htmk(64);
-								htadd(cmd->keywords,zdup(bf),kw_cmd);
+									cmd->keywords = Zhtmk(64);
+								Zhtadd(cmd->keywords, (cmd->ignore ? Zdup(lkwbuf) : Zdup(kwbuf)), kw_cmd);
 							}
 							line = parse_options(syntax,kw_cmd,f,p,1,name,line);
 						} else
-							logerror_2((char *)joe_gettext(_("%s %d: Missing state name\n")),name,line);
+							logerror_2(joe_gettext(_("%s %d: Missing state name\n")),name,line);
 					} else
-						logerror_2((char *)joe_gettext(_("%s %d: Missing string\n")),name,line);
+						logerror_2(joe_gettext(_("%s %d: Missing string\n")),name,line);
 				}
 			}
-		} else if(!zcmp(bf,USTR "noeat")) {
+		} else if(!zcmp(bf,"noeat")) {
 			cmd->noeat = 1;
-		} else if(!zcmp(bf,USTR "mark")) {
+		} else if(!zcmp(bf,"mark")) {
 			cmd->start_mark = 1;
-		} else if(!zcmp(bf,USTR "markend")) {
+		} else if(!zcmp(bf,"markend")) {
 			cmd->stop_mark = 1;
-		} else if(!zcmp(bf,USTR "recolormark")) {
+		} else if(!zcmp(bf,"recolormark")) {
 			cmd->recolor_mark = 1;
 		} else
-			logerror_2((char *)joe_gettext(_("%s %d: Unknown option\n")),name,line);
+			logerror_2(joe_gettext(_("%s %d: Unknown option\n")),name,line);
 	return line;
 }
 
@@ -690,14 +706,13 @@ struct ifstack {
 
 /* Load dfa */
 
-struct high_state *load_dfa(struct high_syntax *syntax)
+static struct high_state *load_dfa(struct high_syntax *syntax)
 {
-	unsigned char name[1024];
-	unsigned char buf[1024];
-	unsigned char bf[256];
-	int clist[256];
-	unsigned char *p;
-	int c;
+	char name[1024];
+	char buf[1024];
+	char bf[256];
+	const char *p;
+	ptrdiff_t c;
 	JFILE *f = 0;
 	struct ifstack *stack=0;
 	struct high_state *state=0;	/* Current state */
@@ -707,18 +722,18 @@ struct high_state *load_dfa(struct high_syntax *syntax)
 	int inside_subr = 0;
 
 	/* Load it */
-	p = (unsigned char *)getenv("HOME");
+	p = getenv("HOME");
 	if (p) {
-		joe_snprintf_2(name,sizeof(name),"%s/.joe/syntax/%s.jsf",p,syntax->name);
+		joe_snprintf_2(name,SIZEOF(name),"%s/.joe/syntax/%s.jsf",p,syntax->name);
 		f = jfopen(name,"r");
 	}
 
 	if (!f) {
-		joe_snprintf_2(name,sizeof(name),"%ssyntax/%s.jsf",JOEDATA,syntax->name);
+		joe_snprintf_2(name,SIZEOF(name),"%ssyntax/%s.jsf",JOEDATA,syntax->name);
 		f = jfopen(name,"r");
 	}
 	if (!f) {
-		joe_snprintf_1(name,sizeof(name),"*%s.jsf",syntax->name);
+		joe_snprintf_1(name,SIZEOF(name),"*%s.jsf",syntax->name);
 		f = jfopen(name,"r");
 	}
 	if (!f) {
@@ -726,14 +741,14 @@ struct high_state *load_dfa(struct high_syntax *syntax)
 	}
 
 	/* Parse file */
-	while(jfgets(buf,1023,f)) {
+	while(jfgets(buf,SIZEOF(buf),f)) {
 		++line;
 		p = buf;
 		c = parse_ws(&p,'#');
 		if (!parse_char(&p, '.')) {
-			if (!parse_ident(&p, bf, sizeof(bf))) {
-				if (!zcmp(bf, USTR "ifdef")) {
-					struct ifstack *st = joe_malloc(sizeof(struct ifstack));
+			if (!parse_ident(&p, bf, SIZEOF(bf))) {
+				if (!zcmp(bf, "ifdef")) {
+					struct ifstack *st = (struct ifstack *)joe_malloc(SIZEOF(struct ifstack));
 					st->next = stack;
 					st->else_part = 0;
 					st->ignore = 1;
@@ -741,7 +756,7 @@ struct high_state *load_dfa(struct high_syntax *syntax)
 					st->line = line;
 					if (!stack || !stack->ignore) {
 						parse_ws(&p,'#');
-						if (!parse_ident(&p, bf, sizeof(bf))) {
+						if (!parse_ident(&p, bf, SIZEOF(bf))) {
 							struct high_param *param;
 							for (param = syntax->params; param; param = param->next)
 								if (!zcmp(param->name, bf)) {
@@ -750,28 +765,28 @@ struct high_state *load_dfa(struct high_syntax *syntax)
 								}
 							st->skip = 0;
 						} else {
-							logerror_2((char *)joe_gettext(_("%s %d: missing parameter for ifdef\n")),name,line);
+							logerror_2(joe_gettext(_("%s %d: missing parameter for ifdef\n")),name,line);
 						}
 					}
 					stack = st;
-				} else if (!zcmp(bf, USTR "else")) {
+				} else if (!zcmp(bf, "else")) {
 					if (stack && !stack->else_part) {
 						stack->else_part = 1;
 						if (!stack->skip)
 							stack->ignore = !stack->ignore;
 					} else
-						logerror_2((char *)joe_gettext(_("%s %d: else with no matching if\n")),name,line);
-				} else if (!zcmp(bf, USTR "endif")) {
+						logerror_2(joe_gettext(_("%s %d: else with no matching if\n")),name,line);
+				} else if (!zcmp(bf, "endif")) {
 					if (stack) {
 						struct ifstack *st = stack;
 						stack = st->next;
 						joe_free(st);
 					} else
-						logerror_2((char *)joe_gettext(_("%s %d: endif with no matching if\n")),name,line);
-				} else if (!zcmp(bf, USTR "subr")) {
+						logerror_2(joe_gettext(_("%s %d: endif with no matching if\n")),name,line);
+				} else if (!zcmp(bf, "subr")) {
 					parse_ws(&p, '#');
-					if (parse_ident(&p, bf, sizeof(bf))) {
-						logerror_2((char *)joe_gettext(_("%s %d: Missing subroutine name\n")),name,line);
+					if (parse_ident(&p, bf, SIZEOF(bf))) {
+						logerror_2(joe_gettext(_("%s %d: Missing subroutine name\n")),name,line);
 					} else {
 						if (!stack || !stack->ignore) {
 							inside_subr = 1;
@@ -780,16 +795,16 @@ struct high_state *load_dfa(struct high_syntax *syntax)
 								this_one = 1;
 						}
 					}
-				} else if (!zcmp(bf, USTR "end")) {
+				} else if (!zcmp(bf, "end")) {
 					if (!stack || !stack->ignore) {
 						this_one = 0;
 						inside_subr = 0;
 					}
 				} else {
-					logerror_2((char *)joe_gettext(_("%s %d: Unknown control statement\n")),name,line);
+					logerror_2(joe_gettext(_("%s %d: Unknown control statement\n")),name,line);
 				}
 			} else {
-				logerror_2((char *)joe_gettext(_("%s %d: Missing control statement name\n")),name,line);
+				logerror_2(joe_gettext(_("%s %d: Missing control statement name\n")),name,line);
 			}
 		} else if (stack && stack->ignore) {
 			/* Ignore this line because of ifdef */
@@ -799,7 +814,7 @@ struct high_state *load_dfa(struct high_syntax *syntax)
 		} else if ((syntax->subr && !this_one) || (!syntax->subr && inside_subr)) {
 			/* Ignore this line because it's not the code we want */
 		} else if(!parse_char(&p, ':')) {
-			if(!parse_ident(&p, bf, sizeof(bf))) {
+			if(!parse_ident(&p, bf, SIZEOF(bf))) {
 
 				state = find_state(syntax,bf);
 
@@ -816,91 +831,80 @@ struct high_state *load_dfa(struct high_syntax *syntax)
 						state->color=color->color;
 					else {
 						state->color=0;
-						logerror_2((char *)joe_gettext(_("%s %d: Unknown class\n")),name,line);
+						logerror_2(joe_gettext(_("%s %d: Unknown class\n")),name,line);
 					}
-					while(parse_ws(&p, '#'), !parse_ident(&p, bf, sizeof(bf))) {
-						if(!zcmp(bf, USTR "comment")) {
+					while(parse_ws(&p, '#'), !parse_ident(&p, bf, SIZEOF(bf))) {
+						if(!zcmp(bf, "comment")) {
 							state->color |= CONTEXT_COMMENT;
-						} else if(!zcmp(bf, USTR "string")) {
+						} else if(!zcmp(bf, "string")) {
 							state->color |= CONTEXT_STRING;
 						} else {
-							logerror_2((char *)joe_gettext(_("%s %d: Unknown context\n")),name,line);
+							logerror_2(joe_gettext(_("%s %d: Unknown context\n")),name,line);
 						}
 					}
 				} else
-					logerror_2((char *)joe_gettext(_("%s %d: Missing color for state definition\n")),name,line);
+					logerror_2(joe_gettext(_("%s %d: Missing color for state definition\n")),name,line);
 			} else
-				logerror_2((char *)joe_gettext(_("%s %d: Missing state name\n")),name,line);
+				logerror_2(joe_gettext(_("%s %d: Missing state name\n")),name,line);
 		} else if(!parse_char(&p, '-')) {
 			/* No. sync lines ignored */
 		} else {
 			c = parse_ws(&p,'#');
 
 			if (!c) {
-			} else if (c=='"' || c=='*' || c=='&') {
+			} else if (c=='"' || c=='*' || c=='&' || c =='%') {
 				if (state) {
-					struct high_cmd *cmd;
-					int delim = 0;
-					if(!parse_field(&p, USTR "*")) {
-						int z;
-						for(z=0;z!=256;++z)
-							clist[z] = 1;
-					} else if(!parse_field(&p, USTR "&")) {
-						delim = 1;
+					struct high_cmd *cmd = mkcmd();
+					if(!parse_field(&p, "*")) {
+						state->dflt = cmd;
+					} else if(!parse_field(&p, "&")) {
+						state->delim = cmd;
+					} else if(!parse_field(&p, "%")) {
+						state->same_delim = cmd;
 					} else {
-						c = parse_string(&p, bf, sizeof(bf));
-						if(c < 0)
-							logerror_2((char *)joe_gettext(_("%s %d: Bad string\n")),name,line);
-						else {
-							int z;
-							int first, second;
-							unsigned char *t = bf;
-							for(z=0;z!=256;++z)
-								clist[z] = 0;
-							while(!parse_range(&t, &first, &second)) {
-								if(first>second)
-									second = first;
-								while(first<=second)
-									clist[first++] = 1;
-							}
+						struct interval *array;
+						ptrdiff_t size;
+						++p;
+						while(*p != '"' && !parse_class(&p, &array, &size)) {
+							rtree_set(&state->rtree, array, size, cmd);
+							/* logmessage_3("Interval add: %x - %x to %p\n", array[0].first, array[0].last, cmd); */
 						}
+						if (*p != '"')
+							logerror_2(joe_gettext(_("%s %d: Bad string\n")),name,line);
+						else
+							++p;
+						/* rtree_show(&state->rtree); */
 					}
 					/* Create command */
-					cmd = mkcmd();
 					parse_ws(&p,'#');
-					if(!parse_ident(&p,bf,sizeof(bf))) {
-						int z;
+					if(!parse_ident(&p,bf,SIZEOF(bf))) {
 						cmd->new_state = find_state(syntax,bf);
 						line = parse_options(syntax,cmd,f,p,0,name,line);
-
-						/* Install command */
-						if (delim)
-							state->delim = cmd;
-						else for(z=0;z!=256;++z)
-							if(clist[z])
-								state->cmd[z]=cmd;
 					} else
-						logerror_2((char *)joe_gettext(_("%s %d: Missing jump\n")),name,line);
+						logerror_2(joe_gettext(_("%s %d: Missing jump\n")),name,line);
 				} else
-					logerror_2((char *)joe_gettext(_("%s %d: No state\n")),name,line);
+					logerror_2(joe_gettext(_("%s %d: No state\n")),name,line);
 			} else
-				logerror_2((char *)joe_gettext(_("%s %d: Unknown character\n")),name,line);
+				logerror_2(joe_gettext(_("%s %d: Unknown character\n")),name,line);
 		}
 	}
 
 	while (stack) {
 		struct ifstack *st = stack;
 		stack = st->next;
-		logerror_2((char *)joe_gettext(_("%s %d: ifdef with no matching endif\n")),name,st->line);
+		logerror_2(joe_gettext(_("%s %d: ifdef with no matching endif\n")),name,st->line);
 		joe_free(st);
 	}
 
 	jfclose(f);
 
+	/* Compile cmaps */
+	build_cmaps(syntax);
+
 	return first;
 }
 
-int syntax_match(struct high_syntax *syntax,unsigned char *name,unsigned char *subr,struct high_param *params)
+static int syntax_match(struct high_syntax *syntax,const char *name,const char *subr,struct high_param *params)
 {
 	struct high_param *syntax_params;
 	if (zcmp(syntax->name,name))
@@ -919,7 +923,7 @@ int syntax_match(struct high_syntax *syntax,unsigned char *name,unsigned char *s
 	return syntax_params == params;
 }
 
-struct high_syntax *load_syntax_subr(unsigned char *name,unsigned char *subr,struct high_param *params)
+struct high_syntax *load_syntax_subr(const char *name,char *subr,struct high_param *params)
 {
 	struct high_syntax *syntax;	/* New syntax table */
 
@@ -931,14 +935,14 @@ struct high_syntax *load_syntax_subr(unsigned char *name,unsigned char *subr,str
 			return syntax;
 
 	/* Create new one */
-	syntax = joe_malloc(sizeof(struct high_syntax));
+	syntax = (struct high_syntax *)joe_malloc(SIZEOF(struct high_syntax));
 	syntax->name = zdup(name);
 	syntax->subr = subr ? zdup(subr) : 0;
 	syntax->params = params;
 	syntax->next = syntax_list;
 	syntax->nstates = 0;
 	syntax->color = 0;
-	syntax->states = joe_malloc(sizeof(struct high_state *)*(syntax->szstates = 64));
+	syntax->states = (struct high_state **)joe_malloc(SIZEOF(struct high_state *)*(syntax->szstates = 64));
 	syntax->ht_states = htmk(syntax->szstates);
 	iz_cmd(&syntax->default_cmd);
 	syntax->default_cmd.reset = 1;
@@ -964,12 +968,17 @@ struct high_syntax *load_syntax_subr(unsigned char *name,unsigned char *subr,str
 	}
 }
 
-struct high_syntax *load_syntax(unsigned char *name)
+struct high_syntax *load_syntax(const char *name)
 {
+	if (!ansi_syntax) {
+		ansi_syntax = (struct high_syntax *)joe_calloc(1, SIZEOF(struct high_syntax));
+		ansi_syntax->name = zdup("ansi");
+	}
+
 	if (!name)
 		return 0;
 
-	if (!zcmp(name, USTR "ansi"))
+	if (!zcmp(name, "ansi"))
 		return ansi_syntax;
 
 	return load_syntax_subr(name,0,0);
