@@ -1,5 +1,5 @@
 /*
- *	UNIX Tty and Process interface
+ *	UNIX TTY and Process interface
  *	Copyright
  *		(C) 1992 Joseph H. Allen
  *
@@ -616,7 +616,7 @@ time_t last_time;
 char ttgetc(void)
 {
         MACRO *m;
-	ptrdiff_t stat;
+	ptrdiff_t mystat;
 	time_t new_time;
 	int flg;
 
@@ -658,11 +658,11 @@ char ttgetc(void)
 	}
 	if (ackkbd != -1) {
 		if (!have) {	/* Wait for input */
-			stat = read(mpxfd, &pack, pack.data - (char *)&pack);
+			mystat = read(mpxfd, &pack, pack.data - (char *)&pack);
 
-			if (pack.size && stat > 0) {
+			if (pack.size && mystat > 0) {
 				joe_read(mpxfd, pack.data, pack.size);
-			} else if (stat < 1) {
+			} else if (mystat < 1) {
 				if (winched || ticked)
 					goto loop;
 				else
@@ -791,7 +791,7 @@ void ttstsz(int fd, ptrdiff_t w, ptrdiff_t h)
 int ttshell(char *cmd)
 {
 	int x, omode = ttymode;
-	int stat= -1;
+	int mystat= -1;
 	const char *s = getenv("SHELL");
 
 	if (!s) {
@@ -805,10 +805,10 @@ int ttshell(char *cmd)
 	if ((x = vfork()) != 0) { /* For AMIGA only  */
 #endif
 		if (x != -1)
-			wait(&stat);
+			wait(&mystat);
 		if (omode)
 			ttopnn();
-		return stat;
+		return mystat;
 	} else {
 		signrm();
 		if (cmd)
@@ -900,7 +900,7 @@ void ttsusp(void)
    select() because joe needs to work on versions of UNIX which predate
    these calls.  Instead, when there is multiple async sources, we use
    helper processes which packetize data from the sources.  A header on each
-   packet indicates the source.  There is no guarentee that packets getting
+   packet indicates the source.  There is no guarantee that packets getting
    written to the same pipe don't get interleaved, but you can reasonable
    rely on it with small packets. */
 
@@ -1134,7 +1134,7 @@ MPX *mpxmk(int *ptyfd, const char *cmd, char **args, void (*func)(void *object,
 	m->die = die;
 	m->dieobj = dieobj;
 
-	/* Acknowledgement pipe */
+	/* Acknowledgment pipe */
 	if (-1 == pipe(fds))
 		return NULL;
 	m->ackfd = fds[1];
@@ -1151,7 +1151,7 @@ MPX *mpxmk(int *ptyfd, const char *cmd, char **args, void (*func)(void *object,
 	if (!(m->kpid = fork())) {
 		/* This process copies data from shell to joe */
 		/* After each packet it sends to joe it waits for
-		   an acknowledgement from joe so that it can not get
+		   an acknowledgment from joe so that it can not get
 		   too far ahead with buffering */
 
 		/* Close joe side of pipes */
@@ -1170,7 +1170,7 @@ MPX *mpxmk(int *ptyfd, const char *cmd, char **args, void (*func)(void *object,
 			/* Close pty (we only need tty) */
 			close(*ptyfd);
 
-			/* All of this stuff is for disassociating ourself from
+			/* All of this stuff is for disassociating our self from
 			   controlling tty (session leader) and starting a new
 			   session.  This is the most non-portable part of UNIX- second
 			   only to pty/tty pair creation. */
