@@ -700,6 +700,7 @@ static int tomatch_char_or_word(BW *bw,int word_delimiter,int c,int f,const char
 				int set0 = utf8_decode_string(set);
 				if (d == set0) {
 					/* ifdef hack */
+					len = 0;
 					if (!joe_isalnum_(p->b->o.charmap, d)) { /* If it's a # in #ifdef, allow spaces after it */
 						sod = p->byte;
 						while ((d = pgetc(p))!=NO_MORE_DATA) {
@@ -708,9 +709,9 @@ static int tomatch_char_or_word(BW *bw,int word_delimiter,int c,int f,const char
 								break;
 							sod = p->byte;
 						}
+						buf[0] = set0;
+						len=1;
 					}
-					buf[0] = set0;
-					len=1;
 					if (joe_isalnum_(p->b->o.charmap, d))
 						goto doit;
 					if (d!=NO_MORE_DATA) {
@@ -726,9 +727,10 @@ static int tomatch_char_or_word(BW *bw,int word_delimiter,int c,int f,const char
 						d=pgetc(p);
 						++col;
 					}
-					if (d!=NO_MORE_DATA)
+					if (d!=NO_MORE_DATA) {
 						prgetc(p);
-					--col;
+						--col;
+					}
 					buf[len]=0;
 					if (is_in_group(set,buf)) {
 						++cnt;
@@ -1323,11 +1325,11 @@ int upgup(W *w, int k)
 	BW *bw;
 	WIND_BW(bw, w);
 	if (menu_above) {
-		if (w->link.prev->watom == &watommenu) {
+		if (w->link.prev->watom == &watommenu && w->link.prev->win == w) {
 			return umpgup(w->link.prev, 0);
 		}
 	} else {
-		if (w->link.next->watom == &watommenu) {
+		if (w->link.next->watom == &watommenu && w->link.next->win == w) {
 			return umpgup(w->link.next, 0);
 		}
 	}
@@ -1351,11 +1353,11 @@ int upgdn(W *w, int k)
 	BW *bw;
 	WIND_BW(bw, w);
 	if (menu_above) {
-		if (w->link.prev->watom == &watommenu) {
+		if (w->link.prev->watom == &watommenu && w->link.prev->win == w) {
 			return umpgdn(w->link.prev, 0);
 		}
 	} else {
-		if (w->link.next->watom == &watommenu) {
+		if (w->link.next->watom == &watommenu && w->link.next->win == w) {
 			return umpgdn(w->link.next, 0);
 		}
 	}
